import logging
import re
from collections import OrderedDict

from inventory_provider import environment
from inventory_provider.db import ims
from inventory_provider.db.ims import InventoryStatus

environment.setup_logging()
logger = logging.getLogger(__name__)

INTERNAL_POP_NAMES = {
    'Cambridge OC',
    'DANTE Lab',
    'GÉANT LAB',
    'GEANT LAB',
    'Amsterdam GEANT Office',
    'Amsterdam GÉANT Office'
}


def lookup_lg_routers(ds):
    pattern = re.compile("vpn-proxy|vrr|taas", re.IGNORECASE)

    def _matching_node(node_):
        if InventoryStatus(node_['InventoryStatusId']) not in [
            InventoryStatus.IN_SERVICE,
            InventoryStatus.PLANNED  # remove once data fully migrated
        ]:
            return False

        if pattern.match(node_['Name']):
            return False

        return True

    site_nav_props = [
        ims.SITE_PROPERTIES['SiteAliases'],
        ims.SITE_PROPERTIES['City'],
        ims.SITE_PROPERTIES['Country']
    ]

    eq_definitions = ds.get_filtered_entities(
        'EquipmentDefinition',
        'Name like mx',
        ims.EQUIP_DEF_PROPERTIES['Nodes'])

    for eq_def in eq_definitions:
        nodes = eq_def['Nodes']

        for node in nodes:
            if not _matching_node(node):
                continue

            site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props,
                                       True)
            city = site['City']

            abbreviation = ''
            try:
                abbreviation = site['SiteAliases'][0]['AliasName']
            except IndexError:
                pass  # no alias - ignore silently

            eq = {
                    'equipment name': node['Name'],
                    'type':
                    'INTERNAL'
                    if site['Name'] in INTERNAL_POP_NAMES
                    else 'CORE',
                    'pop': {
                        'name': site['Name'],
                        'city': city['Name'],
                        'country': city['Country']['Name'],
                        'country code': city['Country']['Abbreviation'],
                        'abbreviation': abbreviation,
                        'longitude': site['Longitude'],
                        'latitude': site['Latitude'],
                    }
                }
            yield(eq)


def lookup_pop_info(ds, hostname):
    site_nav_props = [
        ims.SITE_PROPERTIES['City'],
        ims.SITE_PROPERTIES['SiteAliases'],
        ims.SITE_PROPERTIES['Country']
    ]

    node = ds.get_entity_by_name('Node', hostname)
    if not node:
        return None
    site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props, True)
    city = site['City']
    abbreviation = ''
    try:
        abbreviation = site['SiteAliases'][0]['AliasName']
    except IndexError:
        pass  # no alias - ignore silently
    eq = {
            'equipment-name': node['Name'],
            'status': InventoryStatus(node['InventoryStatusId']).name,
            'pop': {
                'name': site['Name'],
                'city': city['Name'],
                'country': city['Country']['Name'],
                'abbreviation': abbreviation,
                'longitude': site['Longitude'],
                'latitude': site['Latitude'],
            }
        }
    return eq


def otrs_get_customer_company_rows(ds):
    yield ['customer_id', 'name', 'street', 'zip', 'city', 'country', 'url',
           'comments']
    all_cus_comps = set()
    for customer in ds.get_all_entities('Customer'):
        all_cus_comps.add(customer['Name'])
        yield [customer['Name'].replace(' ', ''), customer['Name'],
               '', '', '', '', '', '']
    for vendor in ds.get_all_entities('Vendor'):
        if vendor['Name'] not in all_cus_comps:
            all_cus_comps.add(vendor['Name'])
            yield [vendor['Name'].replace(' ', ''), vendor['Name'],
                   '', '', '', '', '', '']


# TODO - check the rules for validation once model has been confirmed
def _is_valid_customer(cus):
    if not cus['email']:
        return False
    return True


def otrs_get_customer_contacts(ds):

    def _get_customer_id2(t):
        if t['Id'] == 3 or t['Name'] == 'EU NREN':
            return 'OTRS-GEANT-NREN'
        return ''

    for customer in ds.get_all_entities(
            'Customer',
            [
                ims.CUSTOMER_PROPERTIES['CustomerRelatedContacts'],
                ims.CUSTOMER_PROPERTIES['CustomerType']
            ]):

        if customer['CustomerRelatedContacts']:

            for contact in customer['CustomerRelatedContacts']:
                t_customer_user = OrderedDict({
                    'email': contact['Contact']['Mail'],
                    'username': contact['Contact']['Mail'],  # TODO if tal_id is going to be present use that # noqa
                    'customer_id': customer['Name'].replace(' ', ''),
                    'customer_id_2':
                        _get_customer_id2(customer['CustomerType']),
                    'title': '',
                    'firstname': contact['Contact']['Name'],
                    'lastname':
                        ' '.join(filter(None, [
                            contact['Contact']['InFix'],
                            contact['Contact']['LastName']
                        ])),
                    'phone': '',
                    'fax': '',
                    'mobile': '',
                    'street': '',
                    'zip': '',
                    'city': '',
                    'country': '',
                    'comments': '',
                })
                if not _is_valid_customer(t_customer_user):
                    continue
                if contact['Contact']['IsGroup']:
                    t_customer_user['comments'] = \
                        'Organisation imported as a user'

                yield t_customer_user


def otrs_get_vendor_contacts(ds):

    for vrc in ds.get_all_entities(
            'VendorRelatedContact',
            [
                ims.VENDOR_RELATED_CONTACT_PROPERTIES['Vendor'],
                ims.VENDOR_RELATED_CONTACT_PROPERTIES['Contact']
            ]):
        t_customer_user = OrderedDict({
            'email': vrc['Contact']['Mail'],
            'username': vrc['Contact']['Mail'],  # TODO if tal_id is going to be present use that # noqa
            'customer_id': vrc['Vendor']['Name'].replace(' ', ''),
            'customer_id_2': '',
            'title': '',
            'firstname': vrc['Contact']['Name'],
            'lastname':
                ' '.join(filter(None, [
                    vrc['Contact']['InFix'],
                    vrc['Contact']['LastName']
                ])),
            'phone': '',
            'fax': '',
            'mobile': '',
            'street': '',
            'zip': '',
            'city': '',
            'country': '',
            'comments': '',
        })
        if not _is_valid_customer(t_customer_user):
            continue

        yield t_customer_user


def otrs_get_customer_users_rows(ds):
    yield ['email', 'username', 'customer_id', 'customer_id_2', 'title',
           'firstname', 'lastname', 'phone', 'fax', 'mobile', 'street', 'zip',
           'city', 'country', 'comments']

    def get_all_cus_user_rows():
        yielded_customer_emails = set()
        for c in otrs_get_customer_contacts(ds):
            yielded_customer_emails.add(c['email'])
            yield c
        for c in otrs_get_vendor_contacts(ds):
            if c['email'] not in yielded_customer_emails:
                yield c

    sorted_cus = sorted(get_all_cus_user_rows(), key=lambda x: x['email'])
    duplicate_emails = set()
    previous_email = None
    for cu in sorted_cus:
        if cu['email'] == previous_email:
            duplicate_emails.add(previous_email)
        previous_email = cu['email']
    if duplicate_emails:
        logger.error('Duplicate emails found in OTRS customer-user export: '
                       f'{duplicate_emails}')
        # raise KeyError('Duplicate emails found in OTRS customer-user export: '  # noqa
        #                f'{duplicate_emails}')

    for cu in sorted_cus:
        yield list(cu.values())
